#include "mex.h"
#include "math.h"


/*
 * timestwo.c - example found in API guide
 *
 * Computational function that takes a scalar and doubles it.
 *
 * This is a MEX-file for MATLAB.
 * Copyright (c) 1984-2000 The MathWorks, Inc.
 */
 
/* $ Revision: 1.8 $ */

int getIndex(int i, int j, int row)
{
    return row*(j-1)+i-1;
}

//(V(i:n,i)'*a(i:n));
double innerProdM(const double *M, const double *v2, int i, int len)
{int j, start;
double sum=0;
    start = getIndex(i,i,len);
//     mexPrintf("st=%d i=%d len=%d", start, i, len);
    for(j=0;j<len-i+1;j++){
        sum = sum + M[start+j] * v2[j+i-1];
//         mexPrintf("used M[%d]= %f ", start+j, M[start+j]);
    }
    return sum;
}

double innerProdV(const double *v1, const double *v2, int v1_st, int v2_st, int len)
{int j;
double sum=0;
    for(j=0;j<len;j++){
        sum = sum + v1[j+v1_st] * v2[j+v2_st];
    }
    return sum;
}

double norm(double *v1, int len)
{int j;
double sum=0;
    for(j=0;j<len;j++){
        sum = sum+v1[j]*v1[j];
    }
    return sqrt(sum);
}

double sign(double x)
{
    if (x >= 0){
        return 1;
    }
    else{
        return -1;
    }    
       
}


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  double *V, *a, *b, *x, *r, *v; 
  double *re, alpha, in, xnorm;
  int  i, j,n, k, ind;
  mxArray *t1, *t2;
  
  /* Check for proper number of arguments. */
  if(nrhs!=4) {
    mexErrMsgTxt("four input required.");
  } else if(nlhs>1) {
    mexErrMsgTxt("Too many output arguments");
  }
  
  /* The input must be a noncomplex scalar double.*/
//   mrows = mxGetM(prhs[0]);
  n = mxGetM(prhs[1]);
//   if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||
//       !(mrows==1 && ncols==1) ) {
//     mexErrMsgTxt("Input must be a noncomplex scalar double.");
//   }
/* Create matrix for the return argument. */
  k = (int)(mxGetScalar(prhs[2]))-1;
//   mexPrintf("%d   %d  \n",n,k);
  plhs[0] = mxCreateDoubleMatrix(k, 1, mxREAL);
  t1 = mxCreateDoubleMatrix(n-k, 1, mxREAL);
  t2 = mxCreateDoubleMatrix(n-k, 1, mxREAL);
  
  /* Assign pointers to each input and output. */
  V = mxGetPr(prhs[0]);
  a = mxGetPr(prhs[1]);
  b = mxGetPr(prhs[3]);
  x = mxGetPr(t1);
  v = mxGetPr(t2);
  
  plhs[0] = mxCreateDoubleMatrix(k+1, 1, mxREAL);
  r = mxGetPr(plhs[0]);
//   V = mxGetPr(plhs[1]);
//   b = mxGetPr(plhs[2]);
  
  /* Call the timestwo subroutine. */
//   timestwo(re,y,x);
//     for i = 1:k-1,
//         a(i:n) = a(i:n) - 2*V(i:n,i)*(V(i:n,i)'*a(i:n));
//     end 
// for(j=0;j<12;j++){
//     mexPrintf("V[%d]=%f \n",j,V[j]);
// }

// mexPrintf("%d  \n",k);
      for(i=0;i<k;i++){
        in=2*innerProdM(V, a, i+1, n);
// mexPrintf("M inner p = %f  \n",innerProdM(V, a, i+1, n));
        
        for(j=i; j<n; j++){
            ind=getIndex(j+1, i+1, n);
            a[j] = a[j] - in*V[ind];
        }
//         for(j=0;j<n;j++){
//             mexPrintf("a[%d]=%f \n",j,a[j]);
//         }
    }
///


//   x=a(k:n);

 for(j=0;j<n-k;j++){
     x[j]=a[j+k];
 }
//  alpha=-sign(x(1))*norm(x);
    alpha=-sign(x[0])*norm(x, n-k);
//     for(j=0;j<n-k;j++){
//         mexPrintf("x[%d]=%f \n",j,x[j]);
// 
//     }
//     
//  mexPrintf("alpha=%f  \n",alpha);
    
    //x(1)=x(1)-alpha;
    x[0]=x[0]-alpha;
    //v=x/norm(x);
    xnorm=norm(x,n-k);
    for(j=0;j<n-k;j++){
        v[j] = x[j]/xnorm;
    }
    //a(k)=a(k)-2*v(1,1)*(v'*a(k:n));
//     mexPrintf("inner = %f  \n",innerProdV(v, a, 0, k, n-k));
    a[k]=a[k]-2*v[0]*innerProdV(v, a, 0, k, n-k);
//     mexPrintf("a[k] = %f  \n",a[k]);
    //V(k:n,k)=v;
    for(j=0;j<n-k;j++){
        ind=getIndex(j+k+1, k+1, n);
        V[ind]=v[j];
//         mexPrintf("v[%d] = %f  \n",j, v[j]);
    
    }
    //b(k:n) = b(k:n) - 2*v*(v'*b(k:n));
    in=2*innerProdV(v,b,0,k,n-k);
    for(j=k;j<n;j++){
        b[j]=b[j]-in*v[j-k];
//         mexPrintf("b[%d] = %f  \n",j, b[j]);
    }
    //r=a(1:k);
    for(j=0;j<k+1;j++){
        r[j]=a[j];
    }
    
}
